'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _util = require('util');

var _sbEventKit = require('sb-event-kit');

var _vorpal = require('vorpal');

var _vorpal2 = _interopRequireDefault(_vorpal);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var CLI_DELIMITER = _chalk2.default.yellow('♥ motion ♥') + ' ' + _chalk2.default.red('❯') + _chalk2.default.yellow('❯') + _chalk2.default.green('❯');
var WELCOME_MESSAGE = _chalk2.default.red('♥ ♥ ♥ ♥ ♥') + '\t' + _chalk2.default.yellow('Welcome to Motion') + '\t' + _chalk2.default.red('♥ ♥ ♥ ♥ ♥');
var BYE_MESSAGE = _chalk2.default.red('♥ ♥ ♥ ♥ ♥') + '\t' + _chalk2.default.yellow('Bye from Motion') + '\t\t' + _chalk2.default.red('♥ ♥ ♥ ♥ ♥');

class CLI {

  constructor() {
    this.active = false;
    this.instance = (0, _vorpal2.default)();
    this.subscriptions = new _sbEventKit.CompositeDisposable();
  }
  activate() {
    if (process.versions.electron) {
      // Vorpal crashes electron
      return;
    }
    if (this.active) {
      this.instance.show();
      return;
    }

    this.active = true;
    this.instance.delimiter(CLI_DELIMITER);
    this.instance.show();
    this.instance.log(WELCOME_MESSAGE);
  }
  deactivate() {
    this.instance.hide();
  }
  addCommand(name, helpText, callback) {
    var _this = this;

    this.instance.command(name, helpText).action(function (args, keepRunning) {
      var result = callback.call(_this, args);
      if (result && result.constructor.name === 'Promise') {
        result.then(keepRunning, keepRunning);
      } else keepRunning();
    });
  }
  replaceCommand(name, helpText, callback) {
    var command = this.instance.find(name);
    if (command) {
      command.remove();
    }
    this.addCommand(name, helpText, callback);
  }
  log() {
    var contents = [];
    for (var i = 0; i < arguments.length; ++i) {
      var value = arguments[i];
      if (typeof value === 'string') {
        contents.push(value);
      } else if (value && value.constructor.name.endsWith('Error')) {
        contents.push('[' + value.constructor.name + ': ' + value.message + ' ' + value.stack.split('\n')[1].trim() + ']');
      } else {
        contents.push((0, _util.inspect)(value));
      }
    }
    if (this.active) {
      this.instance.log(contents.join(' '));
    } else {
      console.log(contents.join(' '));
    }
  }
  dispose() {
    if (this.active) {
      this.instance.log(BYE_MESSAGE);
      this.instance.hide();
      this.active = false;
    }
    this.subscriptions.dispose();
  }
}
exports.default = CLI;
//# sourceMappingURL=cli.js.map