'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('./fs');

var _sbEventKit = require('sb-event-kit');

var _state = require('./state');

var _state2 = _interopRequireDefault(_state);

var _cli = require('./cli');

var _cli2 = _interopRequireDefault(_cli);

var _error = require('./error');

var _helpers = require('./helpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

class Motion {

  constructor(state, config) {
    var _this = this;

    (0, _assert2.default)(state instanceof _state2.default && (typeof config === 'undefined' ? 'undefined' : _typeof(config)) === 'object', 'Use Motion.create instead of constructor');

    this.cli = new _cli2.default(state, config);
    this.state = state;
    this.config = config;
    this.emitter = new _sbEventKit.Emitter();
    this.watching = false;
    this.subscriptions = new _sbEventKit.CompositeDisposable();

    this.subscriptions.add(this.emitter);
    this.subscriptions.add(this.cli);
    this.cli.onShouldBuild(_asyncToGenerator(function* () {
      yield _this.build(false);
    }));
    this.cli.onShouldDispose(function () {
      _this.dispose();
    });
  }
  exists() {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      return yield (0, _fs.exists)(_this2.config.dataDirectory);
    })();
  }
  watch() {
    var _this3 = this;

    var terminal = arguments.length <= 0 || arguments[0] === undefined ? false : arguments[0];
    return _asyncToGenerator(function* () {
      if (!(yield _this3.exists())) {
        throw new _error.MotionError(_error.ERROR_CODE.NOT_MOTION_APP);
      }

      _this3.state.get().running = true;
      _this3.state.get().process_id = process.pid;
      yield _this3.state.write();

      if (terminal) {
        _this3.cli.activate();
      }
      var pundle = yield (0, _helpers.getPundleInstance)(_this3.state, _this3.config, _this3.cli, terminal, true, function (error) {
        _this3.emitter.emit('did-error', error);
      });
      pundle.listen(_this3.state.get().web_server_port);
      var reloadHook = _this3.cli.onShouldReload(_asyncToGenerator(function* () {
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = pundle.pundle.compilations[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var entry = _step.value;

            entry.clearCache();
            yield entry.compile();
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }
      }));
      var disposable = new _sbEventKit.Disposable(function () {
        _this3.state.get().running = false;
        _this3.state.write();
        _this3.subscriptions.remove(disposable);
        pundle.dispose();
        reloadHook.dispose();
      });

      _this3.subscriptions.add(disposable);
      return disposable;
    })();
  }
  build() {
    var _this4 = this;

    var terminal = arguments.length <= 0 || arguments[0] === undefined ? false : arguments[0];
    return _asyncToGenerator(function* () {
      if (!(yield _this4.exists())) {
        throw new _error.MotionError(_error.ERROR_CODE.NOT_MOTION_APP);
      }
      var compilation = yield (0, _helpers.getPundleInstance)(_this4.state, _this4.config, _this4.cli, terminal, false, function (error) {
        _this4.emitter.emit('did-error', error);
      });
      yield (0, _fs.mkdir)(_path2.default.join(_this4.config.dataDirectory, '_'));
      yield (0, _fs.writeFile)(_path2.default.join(_this4.config.dataDirectory, '_/bundle.js'), (yield compilation.compile()));
    })();
  }
  init() {
    var _this5 = this;

    return _asyncToGenerator(function* () {
      if (yield _this5.exists()) {
        throw new _error.MotionError(_error.ERROR_CODE.ALREADY_MOTION_APP);
      }
      yield (0, _fs.mkdir)(_this5.config.rootDirectory);
      yield (0, _fs.copy)(_path2.default.normalize(_path2.default.join(__dirname, '..', 'template')), _this5.config.rootDirectory);
      _this5.state.write();
    })();
  }
  onDidError(callback) {
    return this.emitter.on('did-error', callback);
  }
  dispose() {
    this.subscriptions.dispose();
  }

  static create(config) {
    return _asyncToGenerator(function* () {
      if (yield (0, _fs.exists)(config.rootDirectory)) {
        config.rootDirectory = yield (0, _fs.realpath)(config.rootDirectory);
      }
      (0, _helpers.fillConfig)(config);
      var state = yield _state2.default.create(_path2.default.join(config.dataDirectory, 'state.json'), _path2.default.join(config.dataDirectory, 'config.json'));
      return new Motion(state, config);
    })();
  }
}

module.exports = Motion;
//# sourceMappingURL=index.js.map